#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'


# import standard library
import json
from datetime import datetime

# import interfaces
from pyams_notify.interfaces import INotification, INotificationHandler
from pyams_security.interfaces import IPrincipalInfo
from pyams_security.interfaces.profile import IPublicProfile
from pyams_skin.interfaces.configuration import IBackOfficeConfiguration
from transaction.interfaces import ITransactionManager

# import packages
from pyams_security.utility import get_principal
from pyams_utils.date import format_datetime
from pyams_utils.request import query_request
from pyams_utils.url import absolute_url
from ws4py.client.threadedclient import WebSocketClient
from zope.interface import implementer
from zope.schema.fieldproperty import FieldProperty


@implementer(INotification)
class Notification(object):
    """Notification object"""

    action = FieldProperty(INotification['action'])
    category = FieldProperty(INotification['category'])
    title = FieldProperty(INotification['title'])
    message = FieldProperty(INotification['message'])
    source = FieldProperty(INotification['source'])
    target = FieldProperty(INotification['target'])
    url = FieldProperty(INotification['url'])
    timestamp = FieldProperty(INotification['timestamp'])
    user_data = FieldProperty(INotification['user_data'])

    def __init__(self, request, **settings):
        self.request = request
        self.context = settings.pop('context', request.context)
        self.action = settings.pop('action', None)
        self.category = settings.pop('category', None)
        self.message = settings.pop('message', None)
        source = settings.pop('source', request.principal.id)
        if IPrincipalInfo.providedBy(source):
            source = source.id
        self.source_id = source
        self.url = settings.pop('url', None)
        self.timestamp = format_datetime(datetime.utcnow())
        self.user_data = settings

    def get_source(self, principal_id=None):
        if principal_id is None:
            principal_id = self.request.principal.id
        principal = get_principal(self.request, principal_id)
        self.source = {'id': principal.id,
                       'title': principal.title}
        profile = IPublicProfile(principal)
        if profile.avatar:
            self.source['avatar'] = absolute_url(profile.avatar, self.request, '++thumb++square:32x32.png')
        configuration = IBackOfficeConfiguration(self.request.root)
        self.title = '{0} - {1}'.format(configuration.short_title, principal.title)

    def get_target(self):
        handler = self.request.registry.queryAdapter(self, INotificationHandler, name=self.category)
        if handler is not None:
            self.target = handler.get_target()

    def send(self):
        self.get_source(self.source_id)
        self.get_target()
        ITransactionManager(self.request.context).get().addAfterCommitHook(notify, kws={'event': self,
                                                                                        'request': self.request})


class NotificationEncoder(json.JSONEncoder):
    """Notification encoder"""

    def default(self, obj):
        if isinstance(obj, Notification):
            return {'action': obj.action,
                    'category': obj.category,
                    'title': obj.title,
                    'message': obj.message,
                    'source': obj.source,
                    'target': obj.target,
                    'url': obj.url,
                    'timestamp': obj.timestamp}
        else:
            return super(NotificationEncoder, self).default(obj)


def notify(status, event, request=None):
    """Send event notification"""
    if not status:  # aborted transaction
        return
    if request is None:
        request = query_request()
    if request is None:
        return
    ws = WebSocketClient('ws://{0}/notify'.format(request.registry.settings.get('pyams_notify.tcp_handler')))
    ws.connect()
    try:
        json_data = json.dumps(event, cls=NotificationEncoder)
        ws.send(json_data)
    finally:
        ws.close()
